function rate_isotope_tracing_GLUT(metabs, params, isotope::Symbol)
    Glucose_media = metabs.Glucose_media_12C + metabs.Glucose_media_13C
    Glucose = metabs.Glucose_12C + metabs.Glucose_13C

    if isotope == :C12
        Rate = (
            (params.GLUT_Vmax * params.GLUT_Conc / params.GLUT_Km_Glucose) *
            (metabs.Glucose_media_12C - (1 / params.GLUT_Keq) * metabs.Glucose_12C) /
            (1 + Glucose_media / params.GLUT_Km_Glucose + Glucose / params.GLUT_Km_Glucose)
        )
    elseif isotope == :C13
        Rate = (
            (params.GLUT_Vmax * params.GLUT_Conc / params.GLUT_Km_Glucose) *
            (metabs.Glucose_media_13C - (1 / params.GLUT_Keq) * metabs.Glucose_13C) /
            (1 + Glucose_media / params.GLUT_Km_Glucose + Glucose / params.GLUT_Km_Glucose)
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_HK1(metabs, params, isotope::Symbol)
    Glucose = metabs.Glucose_12C + metabs.Glucose_13C
    G6P = metabs.G6P_12C + metabs.G6P_13C

    Z = (
        (
            1 +
            (Glucose / params.HK1_K_Glucose) +
            (metabs.ATP / params.HK1_K_a_ATP) +
            (G6P / params.HK1_K_G6P) +
            (G6P / params.HK1_K_a_G6P_cat) +
            (metabs.ADP / params.HK1_K_a_ADP) +
            (params.HK1_β_Glucose_ATP) *
            (Glucose / params.HK1_K_Glucose) *
            (metabs.ATP / params.HK1_K_a_ATP) +
            (Glucose / params.HK1_K_Glucose) * (metabs.ADP / params.HK1_K_a_ADP) +
            (Glucose / params.HK1_K_Glucose) * (G6P / params.HK1_K_a_G6P_cat) +
            (G6P / params.HK1_K_G6P) * (metabs.ADP / params.HK1_K_a_ADP) +
            (G6P / params.HK1_K_G6P) * (G6P / params.HK1_K_a_G6P_cat)
        ) * (1 + (metabs.Phosphate / params.HK1_K_a_Pi)) +
        (1 + (Glucose / params.HK1_K_Glucose) + (G6P / params.HK1_K_G6P)) * (G6P / params.HK1_K_i_G6P_reg)
    )

    if isotope == :C12
        Rate = (
            (
                (
                    params.HK1_Vmax *
                    params.HK1_Conc *
                    (params.HK1_β_Glucose_ATP) *
                    (1 / params.HK1_K_Glucose) *
                    (1 / params.HK1_K_a_ATP) *
                    (1 + (metabs.Phosphate / params.HK1_K_a_Pi))
                ) * (metabs.Glucose_12C * metabs.ATP - metabs.G6P_12C * metabs.ADP / params.HK1_Keq)
            ) / Z
        )
    elseif isotope == :C13
        Rate = (
            (
                (
                    params.HK1_Vmax *
                    params.HK1_Conc *
                    (params.HK1_β_Glucose_ATP) *
                    (1 / params.HK1_K_Glucose) *
                    (1 / params.HK1_K_a_ATP) *
                    (1 + (metabs.Phosphate / params.HK1_K_a_Pi))
                ) * (metabs.Glucose_13C * metabs.ATP - metabs.G6P_13C * metabs.ADP / params.HK1_Keq)
            ) / Z
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_GPI(metabs, params, isotope::Symbol)
    G6P = metabs.G6P_12C + metabs.G6P_13C
    F6P = metabs.F6P_12C + metabs.F6P_13C

    if isotope == :C12
        Rate = (
            (params.GPI_Vmax * params.GPI_Conc / params.GPI_Km_G6P) *
            (metabs.G6P_12C - (1 / params.GPI_Keq) * metabs.F6P_12C) /
            (1 + G6P / params.GPI_Km_G6P + F6P / params.GPI_Km_F6P)
        )
    elseif isotope == :C13
        Rate = (
            (params.GPI_Vmax * params.GPI_Conc / params.GPI_Km_G6P) *
            (metabs.G6P_13C - (1 / params.GPI_Keq) * metabs.F6P_13C) /
            (1 + G6P / params.GPI_Km_G6P + F6P / params.GPI_Km_F6P)
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_PFKP(metabs, params, isotope::Symbol)
    F6P = metabs.F6P_12C + metabs.F6P_13C
    F16BP = metabs.F16BP_12C + metabs.F16BP_13C
    Citrate = metabs.Citrate_12C + metabs.Citrate_13C
    F26BP = metabs.F26BP_12C + metabs.F26BP_13C

    Z_a_cat = (
        1 +
        (F6P / params.PFKP_K_a_F6P) +
        (metabs.ATP / params.PFKP_K_ATP) +
        (F16BP / params.PFKP_K_F16BP) +
        (metabs.ADP / params.PFKP_K_ADP) +
        (F6P / params.PFKP_K_a_F6P) * (metabs.ATP / params.PFKP_K_ATP) +
        (F16BP / params.PFKP_K_F16BP) * (metabs.ADP / params.PFKP_K_ADP)
    )
    Z_i_cat = (
        1 +
        (metabs.ATP / params.PFKP_K_ATP) +
        (F16BP / params.PFKP_K_F16BP) +
        (metabs.ADP / params.PFKP_K_ADP) +
        (F16BP / params.PFKP_K_F16BP) * (metabs.ADP / params.PFKP_K_ADP)
    )
    Z_a_reg = (
        (1 + metabs.Phosphate / params.PFKP_K_Phosphate) *
        (1 + metabs.ADP / params.PFKP_K_a_ADP_reg) *
        (1 + F26BP / params.PFKP_K_a_F26BP)
    )
    Z_i_reg = (
        (1 + metabs.ATP / params.PFKP_K_i_ATP_reg + metabs.Phosphate / params.PFKP_K_Phosphate) *
        (1 + F26BP / params.PFKP_K_i_F26BP) *
        (1 + Citrate / params.PFKP_K_i_Citrate)
    )

    if isotope == :C12
        Rate = ((
            params.PFKP_Vmax *
            params.PFKP_Conc *
            (metabs.F6P_12C * metabs.ATP - metabs.F16BP_12C * metabs.ADP / params.PFKP_Keq) *
            (1 / params.PFKP_K_a_F6P) *
            (1 / params.PFKP_K_ATP) *
            (Z_a_cat^3) *
            (Z_a_reg^4) / ((Z_a_cat^4) * (Z_a_reg^4) + params.PFKP_L * (Z_i_cat^4) * (Z_i_reg^4))
        ))
    elseif isotope == :C13
        Rate = ((
            params.PFKP_Vmax *
            params.PFKP_Conc *
            (metabs.F6P_13C * metabs.ATP - metabs.F16BP_13C * metabs.ADP / params.PFKP_Keq) *
            (1 / params.PFKP_K_a_F6P) *
            (1 / params.PFKP_K_ATP) *
            (Z_a_cat^3) *
            (Z_a_reg^4) / ((Z_a_cat^4) * (Z_a_reg^4) + params.PFKP_L * (Z_i_cat^4) * (Z_i_reg^4))
        ))
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_ALDO(metabs, params, isotope::Symbol)
    F16BP = metabs.F16BP_12C + metabs.F16BP_13C
    GAP = metabs.GAP_12C + metabs.GAP_13C
    DHAP = metabs.DHAP_12C + metabs.DHAP_13C

    if isotope == :C12
        Rate = (
            (params.ALDO_Vmax * params.ALDO_Conc / params.ALDO_Km_F16BP) *
            (metabs.F16BP_12C - (1 / params.ALDO_Keq) * (metabs.DHAP_12C * metabs.GAP_12C)) / (
                1 +
                GAP * DHAP / (params.ALDO_Kd_DHAP * params.ALDO_Km_GAP) +
                DHAP / params.ALDO_Kd_DHAP +
                F16BP * GAP / (params.ALDO_Ki_GAP * params.ALDO_Km_F16BP) +
                F16BP / params.ALDO_Km_F16BP +
                GAP * params.ALDO_Km_DHAP / (params.ALDO_Kd_DHAP * params.ALDO_Km_GAP)
            )
        )
    elseif isotope == :C13
        Rate = (
            (params.ALDO_Vmax * params.ALDO_Conc / params.ALDO_Km_F16BP) *
            (metabs.F16BP_13C - (1 / params.ALDO_Keq) * (metabs.DHAP_13C * metabs.GAP_13C)) / (
                1 +
                GAP * DHAP / (params.ALDO_Kd_DHAP * params.ALDO_Km_GAP) +
                DHAP / params.ALDO_Kd_DHAP +
                F16BP * GAP / (params.ALDO_Ki_GAP * params.ALDO_Km_F16BP) +
                F16BP / params.ALDO_Km_F16BP +
                GAP * params.ALDO_Km_DHAP / (params.ALDO_Kd_DHAP * params.ALDO_Km_GAP)
            )
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_TPI(metabs, params, isotope::Symbol)
    GAP = metabs.GAP_12C + metabs.GAP_13C
    DHAP = metabs.DHAP_12C + metabs.DHAP_13C

    if isotope == :C12
        Rate = (
            (params.TPI_Vmax * params.TPI_Conc / params.TPI_Km_DHAP) *
            (metabs.DHAP_12C - (1 / params.TPI_Keq) * metabs.GAP_12C) /
            (1 + (DHAP / params.TPI_Km_DHAP) + (GAP / params.TPI_Km_GAP))
        )
    elseif isotope == :C13
        Rate = (
            (params.TPI_Vmax * params.TPI_Conc / params.TPI_Km_DHAP) *
            (metabs.DHAP_13C - (1 / params.TPI_Keq) * metabs.GAP_13C) /
            (1 + (DHAP / params.TPI_Km_DHAP) + (GAP / params.TPI_Km_GAP))
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_GAPDH(metabs, params, isotope::Symbol)
    GAP = metabs.GAP_12C + metabs.GAP_13C
    BPG = metabs.BPG_12C + metabs.BPG_13C

    Z_a =
        (
            1 +
            GAP / params.GAPDH_K_GAP * (1 + metabs.Phosphate / params.GAPDH_K_a_Phosphate) +
            BPG / params.GAPDH_K_BPG
        ) * (1 + metabs.NAD / params.GAPDH_K_a_NAD + metabs.NADH / params.GAPDH_K_a_NADH)

    Z_i =
        (1 + metabs.NAD / params.GAPDH_K_i_NAD) * (
            1 +
            GAP / params.GAPDH_K_GAP * (1 + metabs.Phosphate / params.GAPDH_K_i_Phosphate) +
            BPG / params.GAPDH_K_BPG
        ) +
        metabs.NADH / params.GAPDH_K_i_NADH * (
            1 +
            GAP / params.GAPDH_K_GAP * (1 + metabs.Phosphate / params.GAPDH_K_i_Phosphate) +
            BPG / (params.GAPDH_α_i_BPG * params.GAPDH_K_BPG)
        )
    if isotope == :C12
        Rate = ((
            (
                params.GAPDH_Vmax * params.GAPDH_Conc /
                (params.GAPDH_K_GAP * params.GAPDH_K_a_NAD * params.GAPDH_K_a_Phosphate)
            ) *
            Z_a^3 *
            (
                metabs.GAP_12C * metabs.NAD * metabs.Phosphate -
                (1 / params.GAPDH_Keq) * metabs.BPG_12C * metabs.NADH
            ) / (Z_a^4 + params.GAPDH_L * Z_i^4)
        ))
    elseif isotope == :C13
        Rate = ((
            (
                params.GAPDH_Vmax * params.GAPDH_Conc /
                (params.GAPDH_K_GAP * params.GAPDH_K_a_NAD * params.GAPDH_K_a_Phosphate)
            ) *
            Z_a^3 *
            (
                metabs.GAP_13C * metabs.NAD * metabs.Phosphate -
                (1 / params.GAPDH_Keq) * metabs.BPG_13C * metabs.NADH
            ) / (Z_a^4 + params.GAPDH_L * Z_i^4)
        ))
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_PGK(metabs, params, isotope::Symbol)
    BPG = metabs.BPG_12C + metabs.BPG_13C
    ThreePG = metabs.ThreePG_12C + metabs.ThreePG_13C

    if isotope == :C12
        Rate = (
            (params.PGK_Vmax * params.PGK_Conc / (params.PGK_α * params.PGK_K_BPG * params.PGK_K_ADP)) *
            (metabs.BPG_12C * metabs.ADP - (1 / params.PGK_Keq) * (metabs.ThreePG_12C * metabs.ATP)) /
            (
                1 +
                BPG / params.PGK_K_BPG +
                metabs.ADP / params.PGK_K_ADP +
                ThreePG / params.PGK_K_ThreePG +
                metabs.ATP / params.PGK_K_ATP +
                BPG * metabs.ADP / (params.PGK_α * params.PGK_K_BPG * params.PGK_K_ADP) +
                ThreePG * metabs.ATP / (params.PGK_β * params.PGK_K_ThreePG * params.PGK_K_ATP) +
                ThreePG * metabs.ADP / (params.PGK_γ * params.PGK_K_ThreePG * params.PGK_K_ADP)
            )
        )
    elseif isotope == :C13
        Rate = (
            (params.PGK_Vmax * params.PGK_Conc / (params.PGK_α * params.PGK_K_BPG * params.PGK_K_ADP)) *
            (metabs.BPG_13C * metabs.ADP - (1 / params.PGK_Keq) * (metabs.ThreePG_13C * metabs.ATP)) /
            (
                1 +
                BPG / params.PGK_K_BPG +
                metabs.ADP / params.PGK_K_ADP +
                ThreePG / params.PGK_K_ThreePG +
                metabs.ATP / params.PGK_K_ATP +
                BPG * metabs.ADP / (params.PGK_α * params.PGK_K_BPG * params.PGK_K_ADP) +
                ThreePG * metabs.ATP / (params.PGK_β * params.PGK_K_ThreePG * params.PGK_K_ATP) +
                ThreePG * metabs.ADP / (params.PGK_γ * params.PGK_K_ThreePG * params.PGK_K_ADP)
            )
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_PGM(metabs, params, isotope::Symbol)
    ThreePG = metabs.ThreePG_12C + metabs.ThreePG_13C
    TwoPG = metabs.TwoPG_12C + metabs.TwoPG_13C

    if isotope == :C12
        Rate = (
            (params.PGM_Vmax * params.PGM_Conc / params.PGM_Km_ThreePG) *
            (metabs.ThreePG_12C - (1 / params.PGM_Keq) * metabs.TwoPG_12C) /
            (1 + ThreePG / params.PGM_Km_ThreePG + TwoPG / params.PGM_Km_TwoPG)
        )
    elseif isotope == :C13
        Rate = (
            (params.PGM_Vmax * params.PGM_Conc / params.PGM_Km_ThreePG) *
            (metabs.ThreePG_13C - (1 / params.PGM_Keq) * metabs.TwoPG_13C) /
            (1 + ThreePG / params.PGM_Km_ThreePG + TwoPG / params.PGM_Km_TwoPG)
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_ENO(metabs, params, isotope::Symbol)
    TwoPG = metabs.TwoPG_12C + metabs.TwoPG_13C
    PEP = metabs.PEP_12C + metabs.PEP_13C

    if isotope == :C12
        Rate = (
            (params.ENO_Vmax * params.ENO_Conc / params.ENO_Km_TwoPG) *
            (metabs.TwoPG_12C - (1 / params.ENO_Keq) * metabs.PEP_12C) /
            (1 + TwoPG / params.ENO_Km_TwoPG + PEP / params.ENO_Km_PEP)
        )
    elseif isotope == :C13
        Rate = (
            (params.ENO_Vmax * params.ENO_Conc / params.ENO_Km_TwoPG) *
            (metabs.TwoPG_13C - (1 / params.ENO_Keq) * metabs.PEP_13C) /
            (1 + TwoPG / params.ENO_Km_TwoPG + PEP / params.ENO_Km_PEP)
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_PKM2(metabs, params, isotope::Symbol)
    PEP = metabs.PEP_12C + metabs.PEP_13C
    Pyruvate = metabs.Pyruvate_12C + metabs.Pyruvate_13C
    F16BP = metabs.F16BP_12C + metabs.F16BP_13C
    Phenylalanine = metabs.Phenylalanine_12C + metabs.Phenylalanine_13C

    Z_a_cat = (
        1 +
        (PEP / params.PKM2_K_a_PEP) +
        (metabs.ATP / params.PKM2_K_ATP) +
        (metabs.ADP / params.PKM2_K_ADP) +
        (PEP / params.PKM2_K_a_PEP) * (metabs.ADP / params.PKM2_K_ADP) +
        (Pyruvate / params.PKM2_K_Pyruvate) * (metabs.ATP / params.PKM2_K_ATP) +
        (PEP / params.PKM2_K_a_PEP) * (metabs.ATP / params.PKM2_K_ATP) +
        (Pyruvate / params.PKM2_K_Pyruvate) * (metabs.ADP / params.PKM2_K_ADP)
    )
    Z_i_cat = (
        1 +
        (PEP / params.PKM2_K_i_PEP) +
        (metabs.ATP / params.PKM2_K_ATP) +
        (metabs.ADP / params.PKM2_K_ADP) +
        (PEP / params.PKM2_K_i_PEP) * (metabs.ADP / params.PKM2_K_ADP) +
        (Pyruvate / params.PKM2_K_Pyruvate) * (metabs.ATP / params.PKM2_K_ATP) +
        params.PKM2_β_i_PEP_ATP * (PEP / params.PKM2_K_i_PEP) * (metabs.ATP / params.PKM2_K_ATP) +
        (Pyruvate / params.PKM2_K_Pyruvate) * (metabs.ADP / params.PKM2_K_ADP)
    )
    Z_a_reg = ((1 + F16BP / params.PKM2_K_a_F16BP) * (1 + Phenylalanine / params.PKM2_K_a_Phenylalanine))
    Z_i_reg = (1 + Phenylalanine / params.PKM2_K_i_Phenylalanine)

    if isotope == :C12
        Rate = (
            params.PKM2_Conc *
            (metabs.ADP * metabs.PEP_12C - metabs.ATP * metabs.Pyruvate_12C / params.PKM2_Keq) *
            (
                (params.PKM2_Vmax_a * (1.0 / params.PKM2_K_a_PEP) * (1.0 / params.PKM2_K_ADP)) *
                (Z_a_cat^3) *
                (Z_a_reg^4) +
                params.PKM2_L *
                (params.PKM2_Vmax_i * (1.0 / params.PKM2_K_i_PEP) * (1.0 / params.PKM2_K_ADP)) *
                (Z_i_cat^3) *
                (Z_i_reg^4)
            ) / ((Z_a_cat^4) * (Z_a_reg^4) + params.PKM2_L * (Z_i_cat^4) * (Z_i_reg^4))
        )
    elseif isotope == :C13
        Rate = (
            params.PKM2_Conc *
            (metabs.ADP * metabs.PEP_13C - metabs.ATP * metabs.Pyruvate_13C / params.PKM2_Keq) *
            (
                (params.PKM2_Vmax_a * (1.0 / params.PKM2_K_a_PEP) * (1.0 / params.PKM2_K_ADP)) *
                (Z_a_cat^3) *
                (Z_a_reg^4) +
                params.PKM2_L *
                (params.PKM2_Vmax_i * (1.0 / params.PKM2_K_i_PEP) * (1.0 / params.PKM2_K_ADP)) *
                (Z_i_cat^3) *
                (Z_i_reg^4)
            ) / ((Z_a_cat^4) * (Z_a_reg^4) + params.PKM2_L * (Z_i_cat^4) * (Z_i_reg^4))
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_LDH(metabs, params, isotope::Symbol)
    Pyruvate = metabs.Pyruvate_12C + metabs.Pyruvate_13C
    Lactate = metabs.Lactate_12C + metabs.Lactate_13C

    if isotope == :C12
        Rate = (
            (params.LDH_Vmax * params.LDH_Conc / (params.LDH_Km_Pyruvate * params.LDH_Kd_NADH)) *
            (metabs.Pyruvate_12C * metabs.NADH - (1 / params.LDH_Keq) * (metabs.Lactate_12C * metabs.NAD)) /
            (
                1 +
                Pyruvate * params.LDH_Km_NADH / (params.LDH_Kd_NADH * params.LDH_Km_Pyruvate) +
                Lactate * params.LDH_Km_NAD / (params.LDH_Kd_NAD * params.LDH_Km_Lactate) +
                metabs.NADH / params.LDH_Kd_NADH +
                Lactate * metabs.NAD / (params.LDH_Kd_NAD * params.LDH_Km_Lactate) +
                Lactate * metabs.NADH * params.LDH_Km_NAD /
                (params.LDH_Kd_NAD * params.LDH_Kd_NADH * params.LDH_Km_Lactate) +
                Pyruvate * metabs.NADH / (params.LDH_Kd_NADH * params.LDH_Km_Pyruvate) +
                metabs.NAD / params.LDH_Kd_NAD +
                Pyruvate * metabs.NAD * params.LDH_Km_NADH /
                (params.LDH_Kd_NAD * params.LDH_Kd_NADH * params.LDH_Km_Pyruvate)
            )
        )
    elseif isotope == :C13
        Rate = (
            (params.LDH_Vmax * params.LDH_Conc / (params.LDH_Km_Pyruvate * params.LDH_Kd_NADH)) *
            (metabs.Pyruvate_13C * metabs.NADH - (1 / params.LDH_Keq) * (metabs.Lactate_13C * metabs.NAD)) /
            (
                1 +
                Pyruvate * params.LDH_Km_NADH / (params.LDH_Kd_NADH * params.LDH_Km_Pyruvate) +
                Lactate * params.LDH_Km_NAD / (params.LDH_Kd_NAD * params.LDH_Km_Lactate) +
                metabs.NADH / params.LDH_Kd_NADH +
                Lactate * metabs.NAD / (params.LDH_Kd_NAD * params.LDH_Km_Lactate) +
                Lactate * metabs.NADH * params.LDH_Km_NAD /
                (params.LDH_Kd_NAD * params.LDH_Kd_NADH * params.LDH_Km_Lactate) +
                Pyruvate * metabs.NADH / (params.LDH_Kd_NADH * params.LDH_Km_Pyruvate) +
                metabs.NAD / params.LDH_Kd_NAD +
                Pyruvate * metabs.NAD * params.LDH_Km_NADH /
                (params.LDH_Kd_NAD * params.LDH_Kd_NADH * params.LDH_Km_Pyruvate)
            )
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end

function rate_isotope_tracing_MCT(metabs, params, isotope::Symbol)
    Lactate = metabs.Lactate_12C + metabs.Lactate_13C
    Lactate_media = metabs.Lactate_media_12C + metabs.Lactate_media_13C

    if isotope == :C12
        Rate = (
            (params.MCT_Vmax * params.MCT_Conc / params.MCT_Km_Lactate) *
            (metabs.Lactate_12C - (1 / params.MCT_Keq) * metabs.Lactate_media_12C) /
            (1 + Lactate / params.MCT_Km_Lactate + Lactate_media / params.MCT_Km_Lactate)
        )
    elseif isotope == :C13
        Rate = (
            (params.MCT_Vmax * params.MCT_Conc / params.MCT_Km_Lactate) *
            (metabs.Lactate_13C - (1 / params.MCT_Keq) * metabs.Lactate_media_13C) /
            (1 + Lactate / params.MCT_Km_Lactate + Lactate_media / params.MCT_Km_Lactate)
        )
    else
        error("Only :C12 and :C13 arguments are supported")
    end
    return Rate
end
